package com.hz.pm.api.projectdeclared.manage;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.toolkit.Assert;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.hz.pm.api.common.model.constant.BizConst;
import com.hz.pm.api.common.statemachine.util.StateMachineUtil;
import com.hz.pm.api.common.statemachine.util.TenderStateMachineUtil;
import com.hz.pm.api.projectdeclared.model.dto.OperationReq;
import com.hz.pm.api.projectdeclared.model.entity.Operation;
import com.hz.pm.api.projectdeclared.model.entity.Purchase;
import com.hz.pm.api.projectdeclared.model.vo.OperationVO;
import com.hz.pm.api.projectdeclared.service.IOperationService;
import com.hz.pm.api.projectdeclared.service.IPurchaseService;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.enumeration.TenderStatusEnum;
import com.hz.pm.api.projectlib.service.IProjectService;
import com.hz.pm.api.user.security.model.UserInfoDetails;
import com.hz.pm.api.user.util.LoginUserUtil;
import com.ningdatech.basic.exception.BizException;
import com.ningdatech.basic.util.CollUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @Classname OperationManage
 * @Description
 * @Date 2023/7/31 14:48
 * @Author PoffyZhang
 */
@Component
@Slf4j
@RequiredArgsConstructor
public class OperationManage {

    private final IOperationService operationService;
    private final IPurchaseService purchaseService;
    private final IProjectService projectService;
    private final TenderStateMachineUtil tenderStateMachineUtil;
    private final StateMachineUtil stateMachineUtil;

    /**
     * 获取实施详情
     *
     * @param projectId \
     * @return \
     */
    public List<OperationVO> detail(Long projectId) {
        Project project = projectService.getNewProject(projectId);
        Assert.notNull(project, "项目不存在!");

        List<Operation> operations = operationService.list(Wrappers.lambdaQuery(Operation.class)
                .eq(Operation::getProjectCode, project.getProjectCode()));

        return CollUtils.convert(operations, w -> BeanUtil.copyProperties(w, OperationVO.class));
    }

    @Transactional(rollbackFor = Exception.class)
    public synchronized String pushOperation(OperationReq req) {
        UserInfoDetails user = LoginUserUtil.loginUserDetail();
        List<Purchase> purchases = purchaseService.listByProjectId(req.getProjectId());
        Map<Long, Purchase> purchaseMap = CollUtils.listToMap(purchases, Purchase::getId);
        Purchase purchase = purchaseMap.get(req.getBidId());
        if (!TenderStatusEnum.TO_BE_SUBMIT_OPERATION_PLAN.eq(purchase.getStatus())) {
            throw BizException.wrap("该标段不支持填写实施计划");
        }
        Operation pre = operationService.getByBidId(req.getBidId());
        Project project = projectService.getNewProject(req.getProjectId());
        LocalDateTime now = LocalDateTime.now();
        Operation curr = BeanUtil.copyProperties(req, Operation.class);
        if (Objects.nonNull(pre)) {
            curr.setId(pre.getId());
        } else {
            curr.setCreateOn(now);
            curr.setCreateBy(user.getUsername());
        }
        curr.setProjectCode(project.getProjectCode());
        curr.setUpdateOn(now);
        curr.setUpdateBy(user.getUsername());
        operationService.saveOrUpdate(curr);
        long operationCount = operationService.countByBidIds(purchaseMap.keySet());
        if (operationCount == purchaseMap.size()) {
            // 修改项目状态
            stateMachineUtil.pass(project);
            projectService.updateById(project);
        }
        // 修改标段状态
        tenderStateMachineUtil.pass(purchase);
        purchaseService.updateById(purchase);
        return curr.getProjectCode();
    }
}
