package com.hz.pm.api.projectdeclared.model.req;

import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Data;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.List;

/**
 * <p>
 * SubmitPurchaseNoticeReq
 * </p>
 *
 * @author WendyYang
 * @since 11:34 2024/2/22
 */
@Data
@AllArgsConstructor
public class SubmitPurchaseNoticeReq {

    @ApiModelProperty("项目ID")
    @NotNull(message = "请传项目ID")
    private Long projectId;

    @ApiModelProperty("标段信息")
    @NotEmpty(message = "标段信息不能为空")
    private List<PurchaseNoticeTenderDTO> tenders;

    @Data
    public static class PurchaseNoticeTenderDTO {

        @ApiModelProperty("标段ID")
        private Long bidId;

        @ApiModelProperty("标段名称")
        @NotBlank(message = "标段名称不能为空")
        private String bidName;

        @ApiModelProperty("标段预算")
        @NotNull(message = "标段预算不能为空")
        private BigDecimal bidBudget;

        @ApiModelProperty("开标时间")
        @NotNull(message = "开标时间不能为空")
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm")
        private LocalDateTime bidStartTime;

        @ApiModelProperty("投标截止时间")
        @NotNull(message = "投标截止时间不能为空")
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm")
        private LocalDateTime bidEndTime;

        @ApiModelProperty("投标地址")
        @NotBlank(message = "投标地址不能为空")
        private String bidAddress;

        @ApiModelProperty("采购方式")
        @NotBlank(message = "采购方式不能为空")
        private String purchaseMethod;

        @ApiModelProperty("招标文件")
        @NotBlank(message = "招标文件不能为空")
        private String bidDoc;

    }
}
