package com.hz.pm.api.projectlib.handle;

import static com.hz.pm.api.projectlib.model.enumeration.ProjectStatusEnum.*;

import java.util.Arrays;
import java.util.List;
import java.util.Objects;

import com.hz.pm.api.common.model.constant.BizConst;
import com.wflow.workflow.bean.vo.ProcessDetailVO;
import com.wflow.workflow.enums.StepStatusEnum;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.ningdatech.basic.util.CollUtils;
import com.hz.pm.api.common.model.constant.CommonConst;
import com.hz.pm.api.common.statemachine.event.ProjectStatusChangeEvent;
import com.hz.pm.api.projectlib.model.enumeration.ProjectStatusEnum;
import com.hz.pm.api.projectlib.model.entity.ProjectStatusChange;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.service.IProjectStatusChangeService;
import com.hz.pm.api.projectlib.service.IProjectService;

/**
 * 项目初验处理
 *
 * @author CMM
 * @since 2023/02/24 14:35
 */
@Order(10)
@Component
public class ProjectPreliminaryInspectionHandle extends AbstractProcessBusinessHandle {
    private final IProjectStatusChangeService projectStatusChangeService;
    private final IProjectService projectService;

    private Integer order = 10;

    /**
     * 项目状态为：建设中及之后的状态
     */
    private static final List<ProjectStatusEnum> PROJECT_PRELIMINARY_INSPECTION_LIST_STATUS = Arrays.asList(
            UNDER_CONSTRUCTION,
            TO_BE_FINALLY_INSPECTED,
            FINAL_ACCEPTANCE_IS_UNDER_REVIEW,
            FINAL_ACCEPTANCE_REVIEW_FAILED,
            ARCHIVED,
            ACCEPTED
    );

    public ProjectPreliminaryInspectionHandle(IProjectStatusChangeService projectStatusChangeService, IProjectService projectService){
        this.projectStatusChangeService = projectStatusChangeService;
        this.projectService = projectService;
    }

    @Override
    void businessHandle(Project project, List<ProcessDetailVO> processSchedule) {
        ProcessDetailVO processDetailVO = new ProcessDetailVO();
        // 根据项目状态是否在建设中及之后的状态判断项目初验是否开始
        List<Integer> fieldList = CollUtils.fieldList(PROJECT_PRELIMINARY_INSPECTION_LIST_STATUS, ProjectStatusEnum::getCode);
        //查出历史版本的 所有项目ID
        List<Long> allVersionProjectIds = projectService.allVersionProjectIds(project.getProjectCode());
        Integer status = project.getStatus();
        // 项目状态不在建设中及之后的状态
        if (!fieldList.contains(status)){
            processDetailVO.setStepStatus(StepStatusEnum.NOT_START);
            processDetailVO.setProcessName(CommonConst.PROJECT_PRELIMINARY_INSPECTION);
            processSchedule.add(processDetailVO);
            return;
        }
        // 项目状态为建设中或之后的状态
        if (ProjectStatusEnum.UNDER_CONSTRUCTION.getCode().equals(project.getStatus())){
            processDetailVO.setStepStatus(StepStatusEnum.ON_GOING);
        }else {
            processDetailVO.setStepStatus(StepStatusEnum.COMPLETED);
            // 根据初验备案的时间获取
            ProjectStatusChange projectStatusChange = projectStatusChangeService.getOne(Wrappers.lambdaQuery(ProjectStatusChange.class)
                    .in(ProjectStatusChange::getProjectId, allVersionProjectIds)
                    .eq(ProjectStatusChange::getEvent, ProjectStatusChangeEvent.PRELIMINARY_ACCEPTANCE_PUT_ON_RECORD.name())
                    .orderByDesc(ProjectStatusChange::getCreateOn)
                    .last(BizConst.LIMIT_1));
            if(Objects.nonNull(projectStatusChange)){
                processDetailVO.setFinishTime(projectStatusChange.getCreateOn());
            }
        }
        processDetailVO.setProcessName(CommonConst.PROJECT_PRELIMINARY_INSPECTION);
        processSchedule.add(processDetailVO);
    }

    @Override
    Integer getOrder() {
        return order;
    }

    @Override
    void setOrder(Integer order) {
        this.order = order;
    }
}
