package com.hz.pm.api.projectlib.model.enumeration;

import com.hz.pm.api.gov.enumeration.GovProjectStatusEnum;
import lombok.AllArgsConstructor;
import lombok.Getter;
import lombok.NoArgsConstructor;
import org.apache.commons.lang3.StringUtils;

import java.util.Objects;

/**
 * <p>
 * ProjectStatusEnum
 * </p>
 *
 * @author Poffy
 * @since 16:54 2023/2/11
 */
@Getter
@AllArgsConstructor
@NoArgsConstructor
public enum ProjectStatusEnum {
    /**
     * 项目阶段：未立项
     */
    NOT_APPROVED(10000, "未立项", null),
    UNDER_INTERNAL_AUDIT(10001, "单位内部审核中", null),
    UNDER_INTERNAL_AUDIT_NOT_PASS(10002, "单位内部审核不通过", null),
    PENDING_PREQUALIFICATION(10003, "待预审", null),
    JOINT_REVIEW_BY_PROVINCIAL_DEPARTMENTS(10004, "省级部门联审中", null),

    JOINT_REVIEW_BY_PROVINCIAL_DEPARTMENTS_SUCCESS(10018, "省级部门联审通过", null),
    THE_JOINT_REVIEW_OF_PROVINCIAL_DEPARTMENTS_FAILED(10005, "省级部门联审不通过", null),
    PRE_APPLYING(10006, "预审中", null),
    PREQUALIFICATION_FAILED(10007, "预审不通过", null),
    DEPARTMENT_JOINT_REVIEW(10008, "部门联审中", null),
    DEPARTMENT_JOINT_REVIEW_FAILED(10009, "部门联审不通过", null),
    IN_THE_ANNUAL_PLAN(10010, "年度计划中", null),
    BE_SUSPENDED(10011, "被暂缓", null),
    SCHEME_UNDER_REVIEW(10012, "方案评审中", GovProjectStatusEnum.APPROVAL),
    SCHEME_REVIEW_FAILED(10013, "方案评审不通过", null),
    TO_BE_APPROVED(10014, "待立项批复", GovProjectStatusEnum.PENDING),
    TO_BE_DECLARED(10015, "待申报", null),
    PLAN_TO_BE_DECLARED(10016, "方案待申报", null),
    PENDING_PREQUALIFICATION_CHOICE(10017, "待预审选择态", null),
    PREQUALIFICATION_WITHDRAW_CHOICE(10019, "预审中撤回选择态", null),
    APPROVED_AFTER_CHOICE(10020, "立项批复后选择态", null),
    TO_BE_APP_REGISTER(10021, "待应用注册", null),
    /**
     * 信产项目备案
     */
    DECLARED_APPROVED_TO_BE_RECORD(10022, "待立项备案", null),
    DECLARED_APPROVED_RECORD_AUDITING(10023, "立项备案审核中", null),
    DECLARED_APPROVED_RECORD_FAILED(10024, "立项备案审核不通过", null),
    /**
     * 项目阶段：已立项
     */
    PROJECT_APPROVED(20000, "已立项", null),
    TO_BE_PURCHASED(20001, "待采购", GovProjectStatusEnum.APPROVED),
    ON_PURCHASING(20007, "采购中", GovProjectStatusEnum.APPROVED),
    UNDER_CONSTRUCTION(20002, "建设中", null),
    TO_BE_FINALLY_INSPECTED(20003, "待终验", GovProjectStatusEnum.HAS_PRE_INS),
    ON_FINALLY_INSPECTED(20010, "终验中", GovProjectStatusEnum.HAS_PRE_INS),
    TO_BE_FIRST_INSPECTED(20008, "待初验", null),
    ON_FIRST_INSPECTED(20009, "初验中", null),
    FINAL_ACCEPTANCE_IS_UNDER_REVIEW(20004, "终验审核中", null),
    FINAL_ACCEPTANCE_REVIEW_FAILED(20005, "终验审核不通过", null),
    OPERATION(20006, "待开工", GovProjectStatusEnum.PURCHASED),
    /**
     * 项目阶段：已归档
     */
    ARCHIVED(30000, "已归档", null),
    ACCEPTED(30001, "已验收", GovProjectStatusEnum.HAS_FINAL_INS);

    private Integer code;
    private String desc;

    private GovProjectStatusEnum collectionStatus;

    public static String getDesc(Integer code) {
        if (Objects.isNull(code)) {
            return StringUtils.EMPTY;
        }
        for (ProjectStatusEnum t : ProjectStatusEnum.values()) {
            if (code.equals(t.getCode())) {
                return t.desc;
            }
        }
        return StringUtils.EMPTY;
    }

    public static ProjectStatusEnum match(Integer code) {
        if (Objects.isNull(code)) {
            return null;
        }
        for (ProjectStatusEnum t : ProjectStatusEnum.values()) {
            if (code.equals(t.getCode())) {
                return t;
            }
        }
        return null;
    }

    public boolean eq(Integer code) {
        return this.getCode().equals(code);
    }

}
