package com.hz.pm.api.projectlib.service;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.hz.pm.api.common.model.constant.BizConst;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.baomidou.mybatisplus.extension.service.IService;
import com.hz.pm.api.projectlib.model.po.ProjectPO;
import com.hz.pm.api.projectlib.model.req.ProjectListReq;

import java.util.Collection;
import java.util.List;
import java.util.Optional;

/**
 * <p>
 * 服务类
 * </p>
 *
 * @author WendyYang
 * @since 2023-02-03
 */
public interface IProjectService extends IService<Project> {

    default Optional<String> getProjectCode(Long projectId) {
        LambdaQueryWrapper<Project> query = Wrappers.lambdaQuery(Project.class)
                .select(Project::getProjectCode)
                .eq(Project::getId, projectId)
                .last(BizConst.LIMIT_1);
        return Optional.ofNullable(getOne(query))
                .flatMap(w -> Optional.of(w.getProjectCode()));
    }

    default Optional<Long> getNewProjectId(String projectCode) {
        LambdaQueryWrapper<Project> query = Wrappers.lambdaQuery(Project.class)
                .select(Project::getId)
                .eq(Project::getProjectCode, projectCode)
                .orderByDesc(Project::getVersion)
                .last(BizConst.LIMIT_1);
        return Optional.ofNullable(getOne(query))
                .flatMap(w -> Optional.of(w.getId()));
    }

    List<Long> allVersionProjectIds(String projectCode);

    List<Long> allVersionProjectIds(Long projectId);

    Project getNewProject(Long projectId);

    Project getNewConstructProject(Long projectId);

    Project getProjectByCode(String projectCode);

    Page<ProjectPO> pageAllWithPermission(Page<ProjectPO> page, ProjectListReq req);

    Page<ProjectPO> pageLibWithPermission(Page<ProjectPO> page, ProjectListReq req);

    default void reverseNewest(String projectCode, Long projectId) {
        LambdaUpdateWrapper<Project> wrapper = Wrappers.lambdaUpdate(Project.class)
                .set(Project::getNewest, Boolean.FALSE)
                .eq(Project::getNewest, Boolean.TRUE)
                .eq(Project::getProjectCode, projectCode)
                .ne(Project::getId, projectId);
        update(wrapper);
    }

    default List<Project> listNewestByProjectCodes(Collection<String> projectCode) {
        Wrapper<Project> query = Wrappers.lambdaQuery(Project.class)
                .eq(Project::getNewest, Boolean.TRUE)
                .in(Project::getProjectCode, projectCode);
        return list(query);
    }

    default Project getByProjectName(String projectName) {
        LambdaQueryWrapper<Project> query = Wrappers.lambdaQuery(Project.class)
                .eq(Project::getProjectName, projectName)
                .orderByDesc(Project::getVersion)
                .last(BizConst.LIMIT_1);
        return getOne(query);
    }

}
